package org.processmining.parquetxlog.plugins;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.avro.Schema;
import org.apache.avro.SchemaBuilder;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.parquet.avro.AvroParquetWriter;
import org.apache.parquet.hadoop.ParquetWriter;
import org.apache.parquet.hadoop.metadata.CompressionCodecName;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Export Parquet from XLog (minimal)", parameterLabels = { "XLog", "File" }, returnLabels = { }, returnTypes = {})
@UIExportPlugin(description = "Export Parquet from XLog (minimal)", extension = "parquet")
public class ParquetExporterMinimal {
	@PluginVariant(variantLabel = "Export Parquet from XLog (minimal)", requiredParameterLabels = { 0, 1 })
	public void exportXlogToParquet(PluginContext context, XLog log, File file) {
		file.delete();
		exportLog(log, file.getAbsolutePath());
	}
	
	public static void exportLog(XLog log, String targetPath) {
		List<CaseIdActivityClassForExporterMinimal> logToExport = getLogRepresentation(log);
		Path outputPath = new Path(targetPath);
		Schema schema = SchemaBuilder.record("schema").namespace(null).fields().name("caseAAAconceptAAAname").type().nullable().stringType().noDefault().name("conceptAAAname").type().nullable().stringType().noDefault().endRecord();
		  try {
			  ParquetWriter<CaseIdActivityClassForExporterMinimal> writer = AvroParquetWriter
				        .<CaseIdActivityClassForExporterMinimal>builder(outputPath)
				        .withSchema(schema)
				        .withConf(new Configuration())
				        .withCompressionCodec(CompressionCodecName.SNAPPY)
				        .build();
			for (CaseIdActivityClassForExporterMinimal event : logToExport) {
				writer.write(event);
			}
			writer.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	public static List<CaseIdActivityClassForExporterMinimal> getLogRepresentation(XLog log) {
		List<CaseIdActivityClassForExporterMinimal> ret = new ArrayList<CaseIdActivityClassForExporterMinimal>();
		
		int i = 0;
		while (i < log.size()) {
			XTrace trace = log.get(i);
			
			XAttributeMap traceAttributes = trace.getAttributes();
			String caseID = "";
			if (traceAttributes != null && traceAttributes.containsKey("concept:name")) {
				caseID = traceAttributes.get("concept:name").toString();
			}
			else {
				caseID = "caseidx"+i;
			}
			
			for (XEvent event : trace) {
				ret.add(new CaseIdActivityClassForExporterMinimal(caseID, event.getAttributes().get("concept:name").toString()));
			}
			
			i++;
		}
		
		System.out.println(ret.size());
		return ret;
	}
}
