package org.processmining.parquetxlog.plugins;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.avro.Schema;
import org.apache.avro.Schema.Field;
import org.apache.avro.SchemaBuilder;
import org.apache.avro.SchemaBuilder.FieldAssembler;
import org.apache.avro.generic.GenericRecord;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.parquet.avro.AvroParquetReader;
import org.apache.parquet.avro.AvroReadSupport;
import org.apache.parquet.hadoop.ParquetReader;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.parquetxlog.models.IsMinimal;
import org.processmining.parquetxlog.models.ParquetXEventImpl;
import org.processmining.parquetxlog.models.ParquetXLogImpl;
import org.processmining.parquetxlog.models.ParquetXTraceImpl;
import org.processmining.parquetxlog.models.SingletonKeyset;

@Plugin(name = "Import XLog from Parquet file (interactive)", parameterLabels = { "Parquet" }, returnLabels = {
"XLog" }, returnTypes = { XLog.class })
@UIImportPlugin(description = "Import XLog from Parquet file (interactive)", extensions = { "parquet" })
public class ParquetReadLogInteractive extends AbstractImportPlugin {
	public void doAllTest(String filePath) {
		GenericRecord record = readRecordFromFile(filePath);
		List<String> fieldNames = getFieldsNamesFromRecord(record);
		System.out.println(fieldNames);
		List<String> selectedFields = new ArrayList<String>();
		selectedFields.add("case:concept:name");
		selectedFields.add("concept:name");
		selectedFields.add("time:timestamp");
		selectedFields.add("amount");

		Schema finalSchema = buildSchemaFromRecordAndUserSelection(record, selectedFields);
		
		readlog_usingSchema(filePath, finalSchema);
	}
	
	public List<String> getFieldsNamesFromRecord(GenericRecord record) {
		List<String> fieldNames = new ArrayList<String>();
		for (Field f: record.getSchema().getFields()) {
			fieldNames.add(f.name().replaceAll("AAA", ":"));
		}
		fieldNames.remove("concept:name");
		fieldNames.remove("case:concept:name");
		return fieldNames;
	}
	
	public Schema buildSchemaFromRecordAndUserSelection(GenericRecord record, List<String> selectedFields) {
		Schema actualSchema = record.getSchema();
		FieldAssembler<Schema> schemaPrel = SchemaBuilder.record("schema").namespace(null).fields();
		List<String> selectedFields2 = new ArrayList<String>();
		int i = 0;
		while (i < selectedFields.size()) {
			selectedFields2.add(selectedFields.get(i).replaceAll(":", "AAA"));
			i++;
		}
		for (String selectedField : selectedFields2) {
			Field f = actualSchema.getField(selectedField);
			String[] types = f.schema().toString().split("\"");
			String type = types[types.length-2];
			if (type.equals("string")) {
				schemaPrel = schemaPrel.name(selectedField).type().nullable().stringType().noDefault();
			}
			else if (type.equals("double")) {
				schemaPrel = schemaPrel.name(selectedField).type().nullable().doubleType().noDefault();
			}
			else if (type.equals("timestamp-micros")) {
				schemaPrel = schemaPrel.name(selectedField).type().nullable().longType().noDefault();
			}
		}
		Schema schema = schemaPrel.endRecord();
		return schema;
	}
	
	public GenericRecord readRecordFromFile(String filePath) {
		GenericRecord record = null;
		try {
			ParquetReader<GenericRecord> reader = AvroParquetReader.<GenericRecord>builder(new Path(filePath)).build();
			record = reader.read();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return record;
	}
	
	public static XLog readlog_usingSchema(String filePath, Schema schema) {
		IsMinimal.IS_MINIMAL = false;
		ParquetXLogImpl log = new ParquetXLogImpl();
		try {
			Configuration configuration = new Configuration();
			AvroReadSupport<GenericRecord> support = new AvroReadSupport<GenericRecord>();
			support.setRequestedProjection(configuration, schema);
			ParquetReader<GenericRecord> reader = AvroParquetReader.<GenericRecord>builder(support, new Path(filePath)).build();
			Date startDate = new Date();
			SingletonKeyset.destroyInstance();
			List<ParquetXTraceImpl> traces = new ArrayList<ParquetXTraceImpl>();
			String prevCaseID = null;
			while (true) {
				GenericRecord nextRecord = reader.read();
				if (nextRecord == null) {
					break;
				}
				String caseID = nextRecord.get("caseAAAconceptAAAname").toString();
				if (!caseID.equals(prevCaseID)) {
					prevCaseID = caseID;
					log.add(new ParquetXTraceImpl());
				}
				log.get(log.size()-1).add(new ParquetXEventImpl(nextRecord));
			}
			SingletonKeyset.destroyInstance();
			Date endDate = new Date();
		}
		catch (Exception exx) {
			exx.printStackTrace();
		}
		//gc.collect();
		System.out.println("log size = "+log.size());
		return log;
	}
	
	@Override
	@PluginVariant(requiredParameterLabels = { 0 })
	public Object importFile(PluginContext context, File f) throws Exception {
		UIPluginContext context2 = (UIPluginContext) context;
		String filePath = f.getAbsolutePath();
		GenericRecord record = readRecordFromFile(filePath);
		List<String> fieldNames = getFieldsNamesFromRecord(record);
		System.out.println(fieldNames);
		
		StringBuilder fieldNamesStrB = new StringBuilder();
		
		int i = 0;
		while (i < fieldNames.size()) {
			if (i > 0) {
				fieldNamesStrB.append(", ");
			}
			fieldNamesStrB.append(fieldNames.get(i));
			i++;
		}
		
		String fieldNamesStr = fieldNamesStrB.toString();
		
		/*ParquetImportingWizardStep wizardStep = new ParquetImportingWizardStep(fieldNames);
		List<ProMWizardStep<ParquetImportingParameters>> wizStepList = new ArrayList<>();
		wizStepList.add(wizardStep);
		
		ListWizard<ParquetImportingParameters> listWizard = new ListWizard<>(wizStepList);
		ParquetImportingParameters params = ProMWizardDisplay.show(context2, listWizard, new ParquetImportingParameters());
		
		System.out.println("aaa");*/
		
		JOptionPane.showMessageDialog(null, "List of fields available: "+fieldNamesStr);
		
		String input = JOptionPane.showInputDialog(null, "Insert fields to consider (case:concept:name and concept:name already included; comma separated, no spaces): ");
		String[] fields = input.split(",");
		List<String> selectedFields = new ArrayList<String>();
		
		for (String field : fields) {
			if (field.length() > 0) {
				selectedFields.add(field);
			}
		}
		
		selectedFields.add("case:concept:name");
		selectedFields.add("concept:name");
		System.out.println(selectedFields);

		Schema finalSchema = buildSchemaFromRecordAndUserSelection(record, selectedFields);
		System.out.println(finalSchema);
		
		return readlog_usingSchema(filePath, finalSchema);
	}

	protected Object importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
			throws Exception {
		// TODO Auto-generated method stub
		return null;
	}
}
