package org.processmining.parquetxlog.plugins;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.avro.Schema;
import org.apache.avro.SchemaBuilder;
import org.apache.avro.generic.GenericRecord;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.parquet.avro.AvroParquetReader;
import org.apache.parquet.avro.AvroReadSupport;
import org.apache.parquet.hadoop.ParquetReader;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.parquetxlog.models.IsMinimal;
import org.processmining.parquetxlog.models.ParquetXEventImpl;
import org.processmining.parquetxlog.models.ParquetXLogImpl;
import org.processmining.parquetxlog.models.ParquetXTraceImpl;
import org.processmining.parquetxlog.models.SingletonKeyset;

@Plugin(name = "Import XLog from Parquet file (only case id and activity)", parameterLabels = { "Parquet" }, returnLabels = {
"XLog" }, returnTypes = { XLog.class })
@UIImportPlugin(description = "Import XLog from Parquet file (only case id and activity)", extensions = { "parquet" })
public class ParquetReadLogMinimal extends AbstractImportPlugin {
	public static XLog readlog_minimalColumns(String filePath) {
		IsMinimal.IS_MINIMAL = true;
		ParquetXLogImpl log = new ParquetXLogImpl();
		try {
			Configuration configuration = new Configuration();
			Schema schema = SchemaBuilder.record("schema").namespace(null).fields().name("caseAAAconceptAAAname").type().nullable().stringType().noDefault().name("conceptAAAname").type().nullable().stringType().noDefault().endRecord();
			AvroReadSupport<GenericRecord> support = new AvroReadSupport<GenericRecord>();
			support.setRequestedProjection(configuration, schema);
			ParquetReader<GenericRecord> reader = AvroParquetReader.<GenericRecord>builder(support, new Path(filePath)).withConf(configuration).build();
			Date startDate = new Date();
			SingletonKeyset.destroyInstance();
			List<ParquetXTraceImpl> traces = new ArrayList<ParquetXTraceImpl>();
			String prevCaseID = null;
			while (true) {
				GenericRecord nextRecord = reader.read();
				if (nextRecord == null) {
					break;
				}
				String caseID = nextRecord.get("caseAAAconceptAAAname").toString();
				if (!caseID.equals(prevCaseID)) {
					prevCaseID = caseID;
					log.add(new ParquetXTraceImpl());
				}
				log.get(log.size()-1).add(new ParquetXEventImpl(nextRecord));
			}
			SingletonKeyset.destroyInstance();
			Date endDate = new Date();
			System.out.println(endDate.getTime() - startDate.getTime());
		}
		catch (Exception exx) {
			exx.printStackTrace();
		}
		//gc.collect();
		System.out.println("log size = "+log.size());
		return log;
	}

	@Override
	@PluginVariant(requiredParameterLabels = { 0 })
	public Object importFile(PluginContext context, File f) throws Exception {
		return readlog_minimalColumns(f.getAbsolutePath());
	}

	protected Object importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
			throws Exception {
		// TODO Auto-generated method stub
		return null;
	}
}
